import os
import fileutils
from sys import platform
import json


def create_project():
    file = fileutils.root_dir + '/tools/sublime.sublime-project'
    fileutils.createDirForFile(file)

    project_json = None
    if os.path.isfile(file):
        with open(file) as data_file:
            project_json = json.load(data_file)

    if project_json is None:
        project_json = {}
    if 'folders' not in project_json:
        project_json['folders'] = []

    exist = False
    for data in project_json['folders']:
        if 'path' in data:
            exist = exist or data['path'] == fileutils.root_dir

    if not exist:
        project_json['folders'].append({})
        project_json['folders'][-1]['path'] = fileutils.root_dir

    project_json['translate_tabs_to_spaces'] = True

    with open(file, 'w') as outfile:
        json.dump(project_json, outfile, indent=2)


def create_build_system():
    path = ''
    if platform == "win32":
        path = os.getenv('APPDATA') + '\\Sublime Text 3\\Packages\User\\'
    elif platform == 'darwin':
        from AppKit import NSSearchPathForDirectoriesInDomains
        appdata = NSSearchPathForDirectoriesInDomains(14, 1, True)[0] + '/Sublime Text 3'
        path = appdata + '/Packages/User/'
    else:
        exit(-1)
    path += 'Stereo7.sublime-build'
    fileutils.createDirForFile(path)
    open(path, 'w').write('{"shell_cmd": "stereo7 validate -r $project_path/.."}')


def run():
    create_project()
    create_build_system()



import fileutils
import xml.etree.ElementTree as ET
import os

current_file = ''
logs = []

# TODO: add validate spine atlases


def find_file(path):
    search_paths = ['', 'lite/', 'pro/', 'steam/', 'islanddefense', 'islanddefensepro', 'steampunkpro']
    for search_path in search_paths:
        if search_path + path in fileutils.resources:
            return True
    return False


def validate_xmlnode(xmlnode):
    if xmlnode.tag == 'node':
        for name in xmlnode.attrib:
            value = xmlnode.attrib[name]
            validate_property(name, value)
    for child in xmlnode:
        validate_xmlnode(child)
    return True


def validate(file):
    try:
        global current_file
        current_file = file
        tree = ET.parse(fileutils.root_dir + '/Resources/' + file)
        root = tree.getroot()
        validate_xmlnode(root)
    except ET.ParseError as e:
        logs.append('Parsing xml error: [{}]. {}'.format(file, e.message))


def validate_property(name, value):
    if '##' in value:
        return True

    result = True
    msg = '{1} [{2}] not found in Resources'

    if name in ['image', 'imageN', 'imageS', 'imageD']:
        if '::' not in value:
            result = not value or value in fileutils.images or find_file(value)
        else:
            atlas = value[0: value.find('::')]
            frame = value[value.find('::') + 2:]
            result = atlas in fileutils.spriteframes and frame in fileutils.spriteframes[atlas]
            if not result:
                msg = 'spriteframe [{}] not found in atlas [{}]'.format(value, atlas)
    elif value.endswith('.png') or value.endswith('.jpg'):
        if name == 'pair':
            k = value.find(':')
            if k == -1:
                msg = 'Not found divider fro property [pair] = [{}]'.format(value)
                result = False
            icon = value[k + 1:]
            if '::' not in value:
                result = not icon or icon in fileutils.images or find_file(icon)
            else:
                atlas = icon[0: icon.find('::')]
                frame = icon[icon.find('::') + 2:]
                result = atlas in fileutils.spriteframes and frame in fileutils.spriteframes[atlas]
                if not result:
                    msg = 'spriteframe [{}] not found in atlas [{}]'.format(value, atlas)

    if name == 'template':
        result = value in fileutils.xmls or find_file(value)
    if name == 'font' or name == 'fontttf':
        result = value in fileutils.fonts or find_file(value)
    if name == 'spineSkeleton':
        result = value in fileutils.resources or find_file(value)
    if name == 'spineAtlas':
        result = value in fileutils.resources or find_file(value)
        if result:
            result, msg = validate_atlas(value)

    if not result:
        logs.append(('Error in file [{0}]: ' + msg).format(current_file, name, value))

    return result


def validate_atlas(file):
    image = open(fileutils.root_dir + '/Resources/' + file).read().strip().split('\n')[0].strip()
    folder = os.path.dirname(file)
    image = folder + '/' + image
    return image in fileutils.images, 'image [{}] for atlas [{}] not founded'.format(image, file)



import xml.etree.ElementTree as ET
import game
import os
import fileutils

logs = []


class Key:

    def __init__(self):
        self.id = ''
        self.value = {}


class Localization:

    def __init__(self):
        self.lang = ''
        self.keys = {}


localizations = []
all_keys = {}
all_ids = []


def log(msg):
    logs.append(msg)


def validate():
    result = True
    parse()
    count = len(localizations)
    for key in all_keys:
        if len(all_keys[key]) != count:
            without = []
            without.extend(all_ids)
            for id in all_keys[key]:
                without.remove(id)
            log('Error: key [{}] have not locations: [{}]'.format(key, ', '.join(without)))
            result = False
    for localization in localizations:
        for key in localization.keys:
            value = localization.keys[key]
            if not value.strip():
                log('Warning: [{}][{}] is empty'.format(localization.id, key))
    return result


def parse():
    root = ET.parse(fileutils.root_dir + '/Resources/lang/lang.xml').getroot().find('languages')
    for child in root:
        localization = Localization()
        for id in child.attrib:
            localization.id = id
            all_ids.append(id)
            break
        localizations.append(localization)

    for localization in localizations:
        file = fileutils.root_dir + '/Resources/lang/{}.xml'.format(localization.id)
        if not os.path.isdir(file):
            continue
        root = ET.parse(file).getroot().find('dict')
        key = ''
        value = ''
        for i, child in enumerate(root):
            if i % 2 == 0:
                key = child.text
            else:
                value = child.text
                if value is None:
                    value = ''
                localization.keys[key] = value.encode('utf-8')
                if key not in all_keys:
                    all_keys[key] = {}
                all_keys[key][localization.id] = value



win_cmd_build = 'msbuild "@{ROOT}/../../SyndicateBase/proj.win32/SyndicateBase.sln" /p:Configuration=@{CONFIGURATION} /p:Platform=win32 /m'
android_cmd_build = 'gradle @{CONFIGURATION} -p @{ROOT}/proj.android@{APP_KIND_SUFFIX}'
android_cmd_upload_apk = 'gradle publishApkRelease -p @{ROOT}/proj.android@{APP_KIND_SUFFIX}'

ios_cmd_build = 'xcodebuild -workspace @{ROOT}/proj.ios/@{XCODEPROJECT}.xcworkspace archive -archivePath @{XCODEPROJECT}.xcarchive -scheme @{SCHEME} -allowProvisioningUpdates'
ios_cmd_export = 'xcodebuild -exportArchive -archivePath ./@{XCODEPROJECT}.xcarchive -exportPath ./ipa -exportOptionsPlist export.plist -allowProvisioningUpdates'
ios_cmd_upload_inapps = '/Applications/Xcode.app/Contents/Applications/Application\ Loader.app/Contents/itms/bin/iTMSTransporter -m upload -f @{ROOT}/store/inapps.itmsp -u @{APPLE_USER} -p @{APPLE_PASSWORD} -v detailed'

ios_export_plist = '''cat <<EOM > export.plist
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
        <key>teamID</key>
        <string>@{APPLE_TEAM_ID}</string>
        <key>method</key>
        <string>app-store</string>
        <key>uploadSymbols</key>
        <true/>
</dict>
</plist>
EOM
'''

ios_upload_shell = '''
set -ex

# This scripts allows you to upload a binary to the iTunes Connect Store and do it for a specific app_id
# Because when you have multiple apps in status for download, xcodebuild upload will complain that multiple apps are in wait status

# Requires application loader to be installed
# See https://developer.apple.com/library/ios/documentation/LanguagesUtilities/Conceptual/iTunesConnect_Guide/Chapters/SubmittingTheApp.html


IPA_FILE="ipa/@{SCHEME}.ipa"
IPA_FILENAME=$(basename $IPA_FILE)
MD5=$(md5 -q $IPA_FILE)
BYTESIZE=$(stat -f "%z" $IPA_FILE)

TEMPDIR=itsmp
# Remove previous temp
test -d ${TEMPDIR} && rm -rf ${TEMPDIR}
mkdir ${TEMPDIR}
mkdir ${TEMPDIR}/mybundle.itmsp
cp $IPA_FILE ${TEMPDIR}/mybundle.itmsp/$IPA_FILENAME

cat <<EOM > ${TEMPDIR}/mybundle.itmsp/metadata.xml
<?xml version="1.0" encoding="UTF-8"?>
<package version="software4.7" xmlns="http://apple.com/itunes/importer">
<software_assets apple_id="@{APPLE_APP_ID}">
<asset type="bundle">
<data_file>
<file_name>$IPA_FILENAME</file_name>
<checksum type="md5">$MD5</checksum>
<size>$BYTESIZE</size>
</data_file>
</asset>
</software_assets>
</package>
EOM

cp ${IPA_FILE} $TEMPDIR/mybundle.itsmp

USER=@{APPLE_USER}
PASS=@{APPLE_PASSWORD}
/Applications/Xcode.app/Contents/Applications/Application\ Loader.app/Contents/itms/bin/iTMSTransporter -m upload -f ${TEMPDIR} -u "$USER" -p "$PASS" -v detailed
'''



import parse_csv
import diagram


def digit(value):
    value = str(int(value))
    result = ''
    for i in xrange(len(value)):
        if i > 0 and i % 3 == 0:
            result = '\'' + result
        result = value[len(value) - i - 1] + result
    return result


def main(argv):
    file = 'SpentGold.csv'
    if len(argv) > 0:
        file = argv[0]

    table = parse_csv.parse(file)
    table.save('SpentGems_raw.txt')

    count = {}
    sum = 0.0
    index_place = table.columns.index('where')
    index_count = table.columns.index('count')
    for row in table.rows:
        place = row[index_place]
        if place not in count:
            count[place] = 0
        count[place] += int(row[index_count])
        sum += int(row[index_count])

    description = 'Total: ' + digit(sum)
    for place in count:
        gems = digit(count[place])
        percent = round(count[place] / sum * 100, 1)
        description += '\n{}: {} ({}%)'.format(place, gems, percent)

    diagram.build([], 'Spend gems\n%s' % file, 'spend_gems.png',
                  add_text=description)

if __name__ == '__main__':
    main([])
